//**********************************
//   LC meter
//   prosseccer=PIC16F886-I/SP
//   ishizuka.yasuyuki
//   2009.08.30
//**********************************

#include <16F886.h>
//2FE2,3FFF
#fuses HS,NOWDT,PUT,MCLR,NOPROTECT,NOCPD,BROWNOUT,IESO,FCMEN,NOLVP,NODEBUG,NOWRT,BORV40
#use delay(CLOCK = 10000000)

//Ccal(H),Ccal(L),X^[g\,Ccal\,ZLCref\,ZI\,ZG[\
#rom 0x2100={0x03,0xE8,0x3C,0x14,0x14,0x28,0x28}
#rom 0x2110={"LC meter"}				//X^[g\1
#rom 0x2120={"by y.ishizuka"}			//X^[g\2
#rom 0x2130={"Inductor"}				//C_N^[[h̕\
#rom 0x2140={"Not measured"}			//C_N^[[h蒆\
#rom 0x2150={"Capacitor"}				//LpV^[[h̕\
#rom 0x2160={"Out of range"}			//͈͂𒴂ꍇ̕\
#rom 0x2170={"Calibration..."}			//Zs\
#rom 0x2180={"           wait"}			//Zsҋ@\
#rom 0x2190={"           Done"}			//Z\
#rom 0x21A0={"          Error"}			//ZG[\

#bit	T0IF = 0x0b.2	//TIM0I[o[t[rbg
#bit	T1ON = 0x10.0	//TIM1ON/OFFrbg
//#byte	ANSEL = 0x9b

short calflag;					//ZstO
long ccal;						//ZpRfT		//EEPROM
int endf;						//gvtO
int scnt;						//CCP1ݎs
int32 measure_f;				//g
int32 reference_f;				//g		//EEPROM
float reference_c;				//RfT	//EEPROM
float reference_l;				//RC		//EEPROM

void parameter_write(void);		//Zl݁igARfTARCj
void parameter_read(void);		//ZlǂݏoigARfTARCj
void disptime(int n);			//\50mS*n
void io_init(void);				//ݒ
int32 frequency_cnt(void);		//g
void frequency_disp(void);		//g\
void calibration(void);			//Z
void isr_ccp1(void);			//^C}[1
void isr_rb(void);				//O݁iZSWĎj
void l_meter(void);				//RC
void c_meter(void);				//RfT
void write_float_eeprom(long addr,float data);		//float^݁i4oCgj
float read_float_eeprom(long addr);					//float^ǂݏoi4oCgj
void write_int32_eeprom(long addr,int32 data);		//int32^݁i4oCgj
int32 read_int32_eeprom(long addr);					//int32^ǂݏoi4oCgj



////// LCDW[p[^[
#byte	PORTA = 0x05		//|[gAhX
#byte	PORTB = 0x06
#byte	PORTC = 0x07
#define	PORTx 	PORTB
#define	stb		PIN_B3		//E
#define	rs		PIN_B2		//RS
#include  <lcd_kai.c>		//



#int_ccp1
void isr_ccp1(void)				//CCP1
{
	scnt--;
	if(scnt == 0){
		output_low(PIN_A4);
		T1ON = 0;
		endf = 1;
	}
}



#int_ext
void isr_rb(void)				//O
{
	delay_ms(50);
	if(!input(PIN_B0))
	{
		lcd_clear();			//ZJn\
		lcd_eeprom(0x2170);
		lcd_cmd(0xC0);
		lcd_eeprom(0x2180);
		delay_ms(800);
		calflag = 1;
	}
}



void main (void)
{
	io_init();
	
	if(!input(PIN_B0))
	{
		for(;;)
		{
			frequency_disp();
		}
	}

	calflag = 0;
	ccal = ((long)read_eeprom(0x2100)<<8)+(long)read_eeprom(0x2101);

	parameter_read();

	if(!read_eeprom(0x02)==0)		//X^[g\
	{
		lcd_clear();
		lcd_eeprom(0x2110);
		lcd_cmd(0xC0);
		lcd_eeprom(0x2120);
		disptime(read_eeprom(0x02));
	}

	if(!read_eeprom(0x03)==0)		//Ccall\
	{
		lcd_clear();
		printf(lcd_data,"Ccal=%5lupF",ccal);
		lcd_cmd(0xC0);
		printf(lcd_data,"C:%4.0fpF L:%3.0fuH",reference_c,reference_l);
		disptime(read_eeprom(0x03));
	}

	enable_interrupts(INT_EXT);		//OݗL
	
	while(1)
	{
		if(calflag)
		{
			calibration();
		}
		if(input(PIN_A0))
		{
			c_meter();
		}
		else
		{
			l_meter();
		}
	}
}



void write_float_eeprom(long addr,float data)
{
	int *p=&data;
	int i;

	for(i=0;i<4;i++)
	{
		write_eeprom(addr++,*(p++));
	}
}



float read_float_eeprom(long addr)
{
	float data;
	int *p=&data;
	int i;

	for(i=0;i<4;i++)
	{
		*(p++)=read_eeprom(addr++);
	}
	return data;
}



void write_int32_eeprom(long addr,int32 data)
{
	int *p=&data;
	int i;

	for(i=0;i<4;i++)
	{
		write_eeprom(addr++,*(p++));
	}
}



int32 read_int32_eeprom(long addr)
{
	int32 data;
	int *p=&data;
	int i;

	for(i=0;i<4;i++)
	{
		*(p++)=read_eeprom(addr++);
	}
	return data;
}



void parameter_write(void)
{
	write_float_eeprom(0x21F0,reference_c);
	write_float_eeprom(0x21F4,reference_l);
	write_int32_eeprom(0x21F8,reference_f);
}



void parameter_read(void)
{
	reference_c = read_float_eeprom(0x21F0);
	reference_l = read_float_eeprom(0x21F4);
	reference_f = read_int32_eeprom(0x21F8);
}



void io_init(void)
{
	setup_adc(ADC_OFF);				//SăfW^|[gŎgp
	setup_adc_ports(NO_ANALOGS);
	set_tris_a(0x0D);				//RA0:L/CؑSW,RA4:g(sɐ؊),RA2:Cړؑ,RA3:Lg
	set_tris_b(0x01);				//RB0:O(ZSW),RB2-RB7:LCDW[
	set_tris_c(0x00);				//gp
	ext_int_edge(H_TO_L);			//[ANeBuŊO
	output_a(0x00);
	output_b(0x00);
	setup_timer_0(RTCC_EXT_L_TO_H | RTCC_DIV_1);	//^C}[0vXP[[Ȃ
	setup_timer_1(T1_INTERNAL | T1_DIV_BY_8);		//^C}[1vXP[[1/8
	T1ON = 0;										//^C}[1i莞̂ݗLݒj
	set_timer1(0);									//^C}[1obt@NA
	setup_ccp1(CCP_COMPARE_RESET_TIMER);			//CCPW[ݒ
	CCP_1 = 31250;									//100mSݒl
//	CCP_1 = 62500;									//200mSݒl
	enable_interrupts(INT_CCP1);
//	enable_interrupts(INT_EXT);
	enable_interrupts(GLOBAL);
	lcd_init();
	lcd_clear();
}



void disptime(int n)		//*50mS^C}[
{
	int i;
	for(i=0;i<n;i++)
	{
		delay_ms(50);
	}
}



void l_meter(void)
{
	float v,w,x,y,z,lx;

	measure_f = frequency_cnt();
	v = reference_f*1e-3;
	w = measure_f*1e-3;
	y = v * v;
	z = w * w;
	x = (y/z)-1.0;

	lx = x * reference_l;

	lcd_clear();
	lcd_eeprom(0x2130);
	lcd_cmd(0xC0);
	if(measure_f<150)
	{
		lcd_eeprom(0x2140);		//
		return;
	}
	if(lx>5e4 || lx<-20.0)
	{
		lcd_eeprom(0x2160);		//͈͊O
	}
	else
	{
		if(input(PIN_A3))
		{
			if(lx<1e2)
			{
				printf(lcd_data,"Lx=%2.2fuH",lx);	//0.01uH---99.99uH
			}
			else
			{
				if(lx<1e3)
				{
					printf(lcd_data,"Lx=%3.1fuH",lx);  //100.0uH---999.9uH
				}
				else
				{
					if(lx<1e4)
					{
						printf(lcd_data,"Lx=%1.3fmH",lx/1e3);	//1.000mH---9.999mH
					}
					else
					{
						printf(lcd_data,"Lx=%2.2fmH",lx/1e3);	//10.00mH---49.99mH
					}
				}
			}
		}
		else				//nHg
		{
			if(lx<1e0)
			{
				printf(lcd_data,"Lx=%4.0fnH",lx*1e3);	//1nH---999nH
			}
			else
			{
				if(lx<1e2)
				{
					printf(lcd_data,"Lx=%2.2fuH",lx);	//1.00uH---99.99uH
				}
				else
				{
					if(lx<1e3)
					{
						printf(lcd_data,"Lx=%3.1fuH",lx);  //100.0uH---999.9uH
					}
					else
					{
						if(lx<1e4)
						{
							printf(lcd_data,"Lx=%1.3fmH",lx/1e3);	//1.000mH---9.999mH
						}
						else
						{
							printf(lcd_data,"Lx=%2.2fmH",lx/1e3);	//10.00mH---49.99mH
						}
					}
				}
			}
		}
	}
}



void c_meter(void)
{
	float v,w,x,y,z,cx;

	measure_f = frequency_cnt();
	v = reference_f*1e-3;
	w = measure_f*1e-3;
	y = v * v;
	z = w * w;
	x = (y/z)-1.0;

	cx = x * reference_c;

	lcd_clear();
	lcd_eeprom(0x2150);
	lcd_cmd(0xC0);
	if(cx>1.5e6 || cx<-50.0)
	{
		lcd_eeprom(0x2160);		//͈͊O
	}
	else
	{
		if(input(PIN_A2))			//pF/nF
		{
			if(cx<1e3)
			{
				printf(lcd_data,"Cx=%3.1fpF",cx);	//0.1pF---999.9pF
			}
			else
			{
				if(cx<1e4)
				{
					printf(lcd_data,"Cx=%1.3fnF",cx/1e3);	//1.000nF---9.999nF
				}
				else
				{
					if(cx<1e5)
					{
						printf(lcd_data,"Cx=%2.2fnF",cx/1e3);	//10.00nF---99.99nF
					}
					else
					{
						if(cx<1e6)
						{
							printf(lcd_data,"Cx=%3.1fnF",cx/1e3);	//100.0nF---999.9nF
						}
						else
						{
							printf(lcd_data,"Cx=%4.0fnF",cx/1e3);	//1000nF---1499nF
						}
					}
				}		
			}
		}
		else		//pF/uF
		{
			if(cx<1e3)
			{
				printf(lcd_data,"Cx=%3.1fpF",cx);	//0.1pF---999.9pF
			}
			else
			{
				if(cx<1e4)
				{
					printf(lcd_data,"Cx=%4.0fpF",cx);	//1000pF---9999pF
				}
				else
				{
					if(cx<1e5)
					{
						printf(lcd_data,"Cx=0.0%4.0fuF",cx/1e1);	//0.0@1000uF---0.0@9999uF
					}
					else
					{
						if(cx<1e6)
						{
							printf(lcd_data,"Cx=0.%4.0fuF",cx/1e2);	//0.@1000uF---0.@9999uF
						}
						else
						{
							printf(lcd_data,"Cx=%1.3fuF",cx/1e6);	//1.000uF---1.499uF
						}
					}
				}		
			}
		}
	}
}



void calibration(void)
{
	float w,x,y,z;
	
	lcd_clear();			//ZJn\
	lcd_eeprom(0x2170);
	lcd_cmd(0xC0);
	lcd_eeprom(0x2180);
	reference_f = frequency_cnt();		//g擾

	output_high(PIN_A1);				//[ON
	delay_ms(500);
	measure_f = frequency_cnt();		//Zg擾
	output_low(PIN_A1);					//[OFF

	w = reference_f*1e-3;				//float
	x = measure_f*1e-3;
	y = w*w;
	z = x*x;
	reference_c = z/(y-z)*ccal;
	reference_l = 1e8/(3.947842*y*reference_c*1e-3);

	if(reference_c>1200.0||reference_c<800.0||reference_l>120.0||reference_l<80.0)
	{
		lcd_clear();			//ZG[\
		lcd_eeprom(0x2170);
		lcd_cmd(0xC0);
		lcd_eeprom(0x21A0);
		disptime(read_eeprom(0x06));
		delay_ms(100);
		parameter_read();		//ZOɖ߂
		calflag = 0;			//ZtONA
		return;
	}
	if(!read_eeprom(0x04)==0)		//Ccal,Cref,Lref̕\
	{
		lcd_clear();
		lcd_eeprom(0x2170);
		lcd_cmd(0xC0);
		printf(lcd_data,"C:%4.0fpF L:%3.0fuH",reference_c,reference_l);
		disptime(read_eeprom(0x04));
	}
	if(!read_eeprom(0x05)==0)		//ZI\
	{
		lcd_clear();
		lcd_eeprom(0x2170);
		lcd_cmd(0xC0);
		lcd_eeprom(0x2190);
		disptime(read_eeprom(0x05));
	}
	parameter_write();		//ZʂEEPROM
	calflag = 0;			//ZtONA
}



int32 frequency_cnt(void)
{
	int32 fcnt;				//^C}[0I[o[t[JE^[
	int32 freq;				//g

	endf = 0;
	fcnt = 0;
	set_timer0(0);
	T0IF = 0;
	scnt = 5;
	set_timer1(0);
	T1ON = 1;
	delay_cycles(38);		//^C}[
	input(PIN_A4);
	while(!endf)
	{
		if(T0IF)
		{
			fcnt++;
			T0IF = 0;
		}
	}
	freq = ((int32)fcnt * 256 + (int32)get_timer0())*2;
//	freq = (int32)fcnt * 256 + (int32)get_timer0();		//1bXL
	return freq;
}



void frequency_disp(void)
{
	float i;
	i = frequency_cnt()*1e-3;
	lcd_clear();
	printf(lcd_data,"%4.2fKHz",i);
}